#ifdef DEBUG
	#define Assert(e) _Assert(e, #e);
	#define AssertFail() _Assert(false, "<failed>");
#else
	#define Assert(e)
	#define AssertFail()
#endif

namespace Obelisk
{
	typedef unsigned char TByte;
	#ifdef PLATFORM_WINDOWS
		typedef wchar_t TCodeUnit;
	#else
		typedef char TCodeUnit;
	#endif
	extern const TCodeUnit *gEmptyString;



	template <typename T> inline T Min(const T& a, const T& b)
	{
		return a <= b ? a : b;
	}



	template <typename T> inline T Max(const T& a, const T& b)
	{
		return a >= b ? a : b;
	}



	template <typename T> inline void Swap(T& a, T& b)
	{
		T t = a;
		a = b;
		b = t;
	}
}



#include "CException.h"

inline void _Assert(bool aCondition, const char* aMessage)
{
	#if DEBUG
		if (!aCondition)
			throw Obelisk::CException("Assertion failed: %s", aMessage);
	#endif
}



#include "CString.h"
#include "CStringMap.h"
#include "CCollection.h"

#include "IObelisk.h"
#include "IThreadObelisk.h"
#include "IThread.h"
#include "IReader.h"
#include "IWriter.h"
#include "IMainController.h"
#include "CTexture.h"
#include "CVector.h"
#include "CColour.h"

Obelisk::IMainController* CreateMainController(Obelisk::IObelisk* aObelisk);