#include <ext/hash_map>

namespace __gnu_cxx
{
	template<> struct hash<Obelisk::IThread*>
	{
		size_t operator ()(const Obelisk::IThread* aThread) const { return (size_t)aThread; }
	};
}

class CThreadWrapper;

typedef __gnu_cxx::hash_map<Obelisk::IThread*, CThreadWrapper*> TThreadMap;

struct SSoundInfo
{
	SSoundInfo() { }
	SSoundInfo(Obelisk::TByte* aData, int aSize)
	{
		Data = aData;
		Size = aSize;
	}

	Obelisk::TByte *Data;
	int Size;
};

class CApplication : public Obelisk::IObelisk
{
public:
	CApplication();
	~CApplication();

	void Run();

	Display* GetDisplay() { return mDisplay; }

	void SetDirectory(Obelisk::CString aDirectory);
	void SetCaption(Obelisk::CString aCaption);
	void SetSize(int aXSize, int aYSize);

	Obelisk::CString IntToString(int aNumber, int aLeading0s);

	void RunThread(Obelisk::IThread* aThread);
	void NotifyWindow(int aMessage, int aData);

	void StartTimer();
	void StopTimer();

	unsigned int CreateTimer();
	unsigned int Duration(unsigned int aTimer);

	bool MouseButtonDown(bool aRight);
	bool MouseButtonClicked(bool aRight);
	bool MouseButtonReleased(bool aRight);
	void GetCursorPos(int& aCursorX, int& aCursorY);

	bool IsKeyDown(Obelisk::EKey aKey);

	double Random();
	int Random(int aMin, int aMax);

	void LoadTexture(Obelisk::CString aName, bool aTransparent);
	Obelisk::CTexture GetTexture(Obelisk::CString aName);
	void SetLinearFiltering(bool aLinearFiltering);

	void AttachRenderingContext();

	void SetBackgroundColour(float aR, float aG, float aB);
	void BeginFrame();
	void EndFrame();

	void SetSpriteViewport(int aXSize, int aYSize);
	void DrawSprite(Obelisk::CTexture aTexture, float aX, float aY, float aXSize=-1,
			float aYSize=-1, double aXAlign=0, double aYAlign=0);

	void TriColour(const Obelisk::CVector& aV0, const Obelisk::CVector& aV1,
			const Obelisk::CVector& aV2, const Obelisk::CColour& aColour);
	void TriColour(const Obelisk::CVector& aV0, const Obelisk::CColour& aC0,
			const Obelisk::CVector& aV1, const Obelisk::CColour& aC1,
			const Obelisk::CVector& aV2, const Obelisk::CColour& aC2);
	void TriTexture(const Obelisk::CVector& aV0, float aS0, float aT0,
			const Obelisk::CVector& aV1, float aS1, float aT1,
			const Obelisk::CVector& aV2, float aS2, float aT2, Obelisk::CTexture aTexture);

	void SetFont(Obelisk::CStringMap<Obelisk::CTexture> aFont) { mFont = aFont; }
	void DrawText(Obelisk::CString aText, int aX, int aY);

	void LoadSound(Obelisk::CString aName);
	void Sound(Obelisk::CString aName);

	Obelisk::IReader* OpenDataFile(Obelisk::CString aFileName);
	Obelisk::IReader* ReadUserFile(Obelisk::CString aFileName);
	Obelisk::IWriter* WriteUserFile(Obelisk::CString aFileName);

private:
	void Initialise();
	void Destroy();

	Obelisk::IMainController *mMainController;

	bool mDirectorySet;
	Obelisk::CString mDirectory;
	bool mCaptionSet;
	Obelisk::CString mCaption;
	int mWindowXSize;
	int mWindowYSize;

	int NextPowerOf2(int);
	Obelisk::CString GetDataFilePath(Obelisk::CString aFileName);
	bool GetUserFilePath(Obelisk::CString aFileName, bool aCreatePath, Obelisk::CString& aPath);

	Display *mDisplay;
	Window mWindow;
	SDL_Surface *mSurface;

	SDL_TimerID mTimerID;
	bool mTimerTick;
	bool mTimerCallbackActive;
	static Uint32 TimerCallback(Uint32 aInterval, void* aParam);
	void OnTimer();

	bool mMouseLeft;
	bool mMouseLeftLast;
	bool mMouseRight;
	bool mMouseRightLast;

	void OnKeyDown(SDLKey aVirtualKey);
	void OnKeyUp(SDLKey aVirtualKey);
	bool GetKey(SDLKey aVirtualKey, Obelisk::EKey& aKey);
	SDLKey GetSDLKey(Obelisk::EKey aKey);

	void ThreadFinished(Obelisk::IThread* aThread);
	TThreadMap mThreads;

	GLXContext mRenderingContext;
	GLXContext mAltRenderingContext;
	bool mAltAttached;
	bool mLinearFiltering;

	Obelisk::CStringMap<Obelisk::CTexture> mTextures;

	Obelisk::CStringMap<Obelisk::CTexture> mFont;

	Obelisk::CStringMap<SSoundInfo> mSounds;
	bool mAudioInitialised;
	SDL_AudioSpec mAudioSpec;
	static void AudioCallbackProc(void* userdata, Uint8* stream, int len);
	void AudioCallback(Obelisk::TByte* aBuffer, int aSize);
	SSoundInfo mCurrentSound;
	int mCurrentSoundOffset;
};
