﻿#include <hash_map>

class CThreadWrapper;

typedef stdext::hash_map<Obelisk::IThread*, CThreadWrapper*> TThreadMap;

class CApplication : public Obelisk::IObelisk
{
public:
	CApplication(HINSTANCE aInstance);
	~CApplication();

	void Run();

	HWND GetWindow() { return mWindow; }

	bool CancelSynchronousIo(HANDLE aThread);

	void SetDirectory(Obelisk::CString aDirectory);
	void SetCaption(Obelisk::CString aCaption);
	void SetSize(int aXSize, int aYSize);

	Obelisk::CString IntToString(int aNumber, int aLeading0s);

	void RunThread(Obelisk::IThread* aThread);
	void NotifyWindow(int aMessage, int aData);

	void StartTimer();
	void StopTimer();
	HANDLE mWaitableTimer;

	unsigned int CreateTimer();
	unsigned int Duration(unsigned int aTimer);

	bool MouseButtonDown(bool aRight);
	bool MouseButtonClicked(bool aRight);
	bool MouseButtonReleased(bool aRight);
	void GetCursorPos(int& aCursorX, int& aCursorY);

	bool IsKeyDown(Obelisk::EKey aKey);

	double Random();
	int Random(int aMin, int aMax);

	void LoadTexture(Obelisk::CString aName, bool aTransparent);
	Obelisk::CTexture CreateTexture(Obelisk::CString aName, unsigned int* aData, int aXSize, int aYSize);
	Obelisk::CTexture GetTexture(Obelisk::CString aName);
	void SetLinearFiltering(bool aLinearFiltering);

	void AttachRenderingContext();

	void SetBackgroundColour(float aR, float aG, float aB);
	void BeginFrame();
	void EndFrame();
	void SetAlpha(float aAlpha) { mAlpha = aAlpha; }

	void SetSpriteViewport(int aXSize, int aYSize);
	void DrawSprite(Obelisk::CTexture aTexture, int aX, int aY, int aXSize=-1,
			int aYSize=-1, float aXAlign=0, float aYAlign=0);

	void TriColour(const Obelisk::CVector& aV0, const Obelisk::CVector& aV1,
			const Obelisk::CVector& aV2, const Obelisk::CColour& aColour);
	void TriColour(const Obelisk::CVector& aV0, const Obelisk::CColour& aC0,
			const Obelisk::CVector& aV1, const Obelisk::CColour& aC1,
			const Obelisk::CVector& aV2, const Obelisk::CColour& aC2);
	void TriTexture(const Obelisk::CVector& aV0, float aS0, float aT0,
			const Obelisk::CVector& aV1, float aS1, float aT1,
			const Obelisk::CVector& aV2, float aS2, float aT2, Obelisk::CTexture aTexture);

	Obelisk::CStringMap<Obelisk::CTexture> LoadFontImage(Obelisk::CString aName);
	void SetFont(Obelisk::CStringMap<Obelisk::CTexture> aFont) { mFont = aFont; }
	void DrawText(Obelisk::CString aText, int aX, int aY);

	void LoadSound(Obelisk::CString aName);
	void Sound(Obelisk::CString aName);

	Obelisk::IReader* OpenDataFile(Obelisk::CString aFileName);
	Obelisk::IReader* ReadUserFile(Obelisk::CString aFileName);
	Obelisk::IWriter* WriteUserFile(Obelisk::CString aFileName);

private:
	HINSTANCE mInstance;

	BOOL (WINAPI *mCancelSynchronousIo)(HANDLE hThread);

	void Initialise();

	bool mDirectorySet;
	Obelisk::CString mDirectory;
	bool mCaptionSet;
	Obelisk::CString mCaption;
	int mWindowXSize;
	int mWindowYSize;
	HWND mWindow;
	static LRESULT CALLBACK MainWndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
	void WindowDestroyed();
	bool OnSetCursor();
	DWORD mMainThreadID;

	#ifdef DEBUG
		bool mFrame;
	#endif

	Obelisk::CString GetDataFilePath(Obelisk::CString aFileName);
	bool GetUserFilePath(Obelisk::CString aFileName, bool aCreatePath, Obelisk::CString& aPath);

	bool mTimerStarted;
	bool mTimerCallbackActive;
	void OnTimer();

	bool mMouseLeft;
	bool mMouseLeftLast;
	bool mMouseRight;
	bool mMouseRightLast;

	void OnKeyDown(int aVirtualKey);
	void OnKeyUp(int aVirtualKey);
	bool GetKey(int aVirtualKey, Obelisk::EKey& aKey);
	int GetVirtualKey(Obelisk::EKey aKey);

	HDC mDC;
	HGLRC mRenderingContext;
	HGLRC mAltRenderingContext;
	bool mAltAttached;
	bool mLinearFiltering;
	float mAlpha;

	TThreadMap mThreads;
	void ThreadFinished(Obelisk::IThread* aThread);

	Obelisk::IMainController *mMainController;

	int NextPowerOf2(int);
	Obelisk::CStringMap<Obelisk::CTexture> mTextures;
	Obelisk::CStringMap<Obelisk::CStringMap<Obelisk::CTexture> > mFonts;

	Obelisk::CStringMap<Obelisk::CTexture> mFont;

	Obelisk::CStringMap<Obelisk::TByte*> mSounds;
};