﻿#include <stack>
#include "Obelisk.h"

#include "CGameController.h"
#include "CGameState.h"
#include "CIsland.h"

using namespace std;
using namespace Obelisk;



CGameState::CGameState(IObelisk* aObelisk, CGameController* aOwner)
{
	mObelisk = aObelisk;
	mOwner = aOwner;

	mRank = 1;
	mNumYourIslands = 0;
	mOil = 0;
	mOilDelta = 0;
	mOilDiff = 0;
	mHeliOil = 0;
	mRankOil = 0;
	mHighestLevel = 0;

	mBoatX = -10;
	mBoatY = 0;
	mBoatDir = 1;
	mBoatFuel = 2500;
	mBoatMoveMaxSteps = -1;

	mNumHelis = 5;

	mFirstIslandWon = true;

	GenerateMap();
}



CGameState::~CGameState()
{
	foreach (mIslands)
		delete mIslands.Current();
}



int CGameState::GetHeliOilMax()
{
	return mRank * 25;
}



int CGameState::GetRankOilMax()
{
	return mRank * 25 * 10;
}



void CGameState::Update()
{
	mOilDiff += mOilDelta;
	while (mOilDiff >= 600)
	{
		mOil++;
		mOilDiff -= 600;
		if (mHeliOil < mRank * 25)
			mHeliOil++;
		if (mRankOil < mRank * 25 * 10)
			mRankOil++;
	}
	if (mHeliOil == mRank * 25 && mNumHelis < mRank * 2 + 5)
	{
		mHeliOil = 0;
		mNumHelis++;
	}
	if (mRankOil == mRank * 25 * 10 && mHighestLevel >= mRank)
	{
		mRankOil = 0;
		mRank++;
		if (mRank == 2)
			mOwner->FirstRank();
	}

	if (mBoatMoveMaxSteps != -1)
	{
		mBoatMoveStep++;
		GetBoatPos(mBoatX, mBoatY);
		if (mBoatDestX < mBoatX)
			mBoatDir = 0;
		if (mBoatDestX > mBoatX)
			mBoatDir = 1;
		mBoatFuel -= 6;
		if (mBoatFuel <= 0)
			mOwner->GameOver(SL("Your Ship Ran Out Of Fuel"), 398);
		if (mBoatMoveStep == mBoatMoveMaxSteps)
			mBoatMoveMaxSteps = -1;
	}
}



void CGameState::WinIsland(CIsland* aIsland)
{
	if (mFirstIslandWon)
	{
		mFirstIslandWon = false;
		mOwner->FirstIslandWon();
	}
	Refuel();
	aIsland->SetOwner(0);
	mNumYourIslands++;
	mOilDelta += aIsland->GetOilRate();
	if (aIsland->GetLevel() > mHighestLevel)
	{
		mHighestLevel = aIsland->GetLevel();
		mOwner->LevelWon(mHighestLevel);
	}
}



void CGameState::GetBoatPos(float& aBoatX, float& aBoatY)
{
	float Distance = (float)mBoatMoveStep / mBoatMoveMaxSteps;
	aBoatX = mBoatStartX * (1 - Distance) + mBoatDestX * Distance;
	aBoatY = mBoatStartY * (1 - Distance) + mBoatDestY * Distance;
}



void CGameState::MoveBoat(float aX, float aY)
{
	mBoatStartX = mBoatX;
	mBoatStartY = mBoatY;
	mBoatDestX = aX;
	mBoatDestY = aY;
	mBoatMoveStep = 0;
	mBoatMoveMaxSteps = (int)(sqrt(pow(mBoatDestX - mBoatStartX, 2) + pow(mBoatDestY -
			mBoatStartY, 2)) * 2);
}



void CGameState::Refuel()
{
	mBoatFuel = 2500;
}



void CGameState::GenerateMap()
{
	for (;;)
	{
		CCollection<CIsland> Islands = GenerateIslands();

		CCollection<CIsland> SeenIslands;
		CCollection<CIsland> ExistingIslands = Islands;
		CIsland *Island;
		foreach (Islands)
		{
			Island = Islands.Current();
			if (Island != NULL)   // this will never be NULL, but the compiler complains otherwise
				break;
		}
		stack<CIsland*> Stack;
		Stack.push(Island);
		while (!Stack.empty())
		{
			Island = Stack.top();
			Stack.pop();
			if (SeenIslands.Contains(Island))
				continue;
			SeenIslands.Add(Island);
			ExistingIslands.Remove(Island);
			foreach (ExistingIslands)
			{
				if (pow((float)(ExistingIslands->GetX() - Island->GetX()), 2) +
						pow((float)(ExistingIslands->GetY() - Island->GetY()), 2) < 150 * 150)
					Stack.push(ExistingIslands.Current());
			}
		}
		if (SeenIslands.GetCount() == Islands.GetCount())
		{
			mIslands = Islands;
			break;
		}
	}
}



CCollection<CIsland> CGameState::GenerateIslands()
{
	CCollection<CIsland> Islands;
	Islands.Add(new CIsland(20, 20, 1, 1, 1 * 2 + 1));
	Islands.Add(new CIsland(980, 980, 1, 11, 11 * 2 + 1));
	for (int i = 0; i < 80; i++)
	{
		int X;
		int Y;
		for (;;)
		{
			X = mObelisk->Random(0, 1000);
			Y = mObelisk->Random(0, 1000);
			bool OK = true;
			foreach (Islands)
			{
				if (pow((float)(Islands->GetX() - X), 2) + pow((float)(Islands->GetY() - Y),
						2) < 50 * 50)
				{
					OK = false;
					break;
				}
			}
			if (OK)
				break;
		}
		float Distance = sqrt(pow((float)X - 20, 2) + pow((float)Y - 20, 2));
		int Level = (int)(Distance / 160) + 1;
		if (Level > 9)   // this shouldn't happen, but just in case
			Level = 9;
		Islands.Add(new CIsland(X, Y, 1, Level, Level * 2 + 1));
	}
	return Islands;
}